'Diebold-Mariano forecast evaluation test (JBES 1995);
'inputs:  two equation objects, and a forecast sample.
'Output is a spool with a results table and graphs of the forecast errors

'step 1:
'forecast from each equation over the sample period.
!selection = 1
%eq1 = ""
%eq2 = ""
%smpl = ""
@uidialog("edit", %eq1, "Enter name of equation 1", "edit", %eq2, "Enter name of equation 2", "edit", %smpl, "Enter a forecast sample", "radio",!selection,"Please select a loss function:", "Squared Absolute")

'error check for object existance and type
if @isobject(%eq1) = 0 then  
	%error = %eq1 + " is not an equation in your workfile"
	@uiprompt(%error)
	stop
	else
		if @neqna({%eq1}.@type,"EQUATION") = 1 then
			%error = %eq1 + " is not an equation in your workfile"
			@uiprompt(%error)
			stop
		endif
endif

'error check for object existance and type
if @isobject(%eq2) = 0 then  
	%error = %eq2+ " is not an equation in your workfile"
	@uiprompt(%error)
	stop
	else
		if @neqna({%eq2}.@type,"EQUATION") = 1 then
			%error = %eq2 + " is not an equation in your workfile"
			@uiprompt(%error)
			stop
		endif
endif

'check to make sure the actual values of the dependent variable exist and the equations have the same dependent variable
%vars1 = {%eq1}.@varlist
%vars2 = {%eq2}.@varlist
%dep1 = @word(%vars1,1)  'get dependent variable
%dep2 = @word(%vars2,1)  'get dependent variable

'save sample
%origsmpl = @pagesmpl

'set sample
smpl {%smpl}

%NDEP = @getnextname("NDEP")
series {%NDEP} = {%dep1}

'checking for the existence of actual values inside the dependent variable over the forecast period
if @obssmpl <> {%NDEP}.@obs then
	%msg = "Actual values of dependent variable must exist over the forecast period"
	@uiprompt(%msg)
	delete {%ndep}
	smpl {%origsmpl}
	stop
endif

delete {%ndep}

If %dep1 <> %dep2 then
%msg = "Equations must have the same dependent variable"
@uiprompt(%msg)
stop
endif

'Checking to make sure only simple samples are being used
if @wcount(%smpl) > 2 then
	%msg = "Only simple forecast samples are supported"
	@uiprompt(%msg)
	stop
endif

'perform the forecasts - creating two series, forecast1 and forecast2
%forecast1 = @getnextname("forecast1")
%forecast2 = @getnextname("forecast2")

{%eq1}.forecast {%forecast1}
{%eq2}.forecast {%forecast2}

'generating forecast errors
%FE1 = @getnextname("FE1")
%FE2 = @getnextname("FE2")

series {%FE1} = {%dep1} - {%forecast1}
series {%FE2} = {%dep2} - {%forecast2}

%Ld = @getnextname("LD")

'Step 2:
'calculate Ld, find mean and varialce
if !selection = 1 then
	series {%Ld} = {%FE1}^2-{%FE2}^2
else 
	series {%Ld} = @abs({%FE1}) - @abs({%FE2})
endif
!Ldiffm = @mean({%Ld})
!Ldiffv = @var({%Ld})


'Step 3:
'Calculate covariances

'calculate number of observations in forecast sample
!t = @obssmpl

!DMstat = !Ldiffm / @sqrt((1/!T)*!Ldiffv )

!pval = 2*@cnorm(-@abs(!DMstat))

'd(noerr) a
'table a
'a(1,1) = !dmstat

if !selection = 1 then
	%lossfunc = "Squared"
else
	%lossfunc = "Absolute"
endif

'Results
'Presented in a Table
%tn = @getnextname("DMtest")
Table(9,2) {%tn}
{%tn}.setwidth(1) 18
{%tn}.setwidth(2) 10
{%tn}.setjust(a1:a9) left
!rowcounter=1
{%tn}(!rowcounter,1) = "Diebold-Mariano Forecast Eval. Test"
!rowcounter = !rowcounter + 1
%datestring = "Date: " + @datestr(@now,"MM/DD/YY") + "   Time: " + @datestr(@now,"HH:MM")
{%tn}(!rowcounter,1) = %datestring
!rowcounter=!rowcounter+1
%samplestr = "Equation 1: "  + @upper(%eq1)
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
%samplestr = "Equation 2: " + @upper(%eq2)
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
%samplestr = "Forecast Sample: "  + @upper(%smpl)
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
%samplestr = "Loss function: "  + %lossfunc
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1
'---------------------------
{%tn}(!rowcounter,1) = "DM Test Statistic: "
{%tn}(!rowcounter,2) = !DMstat
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "P-value: "
{%tn}(!rowcounter,2) = !pval
!rowcounter = !rowcounter + 1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

'creating a spool with table and graphs
%sp = @getnextname("DMResults")
spool {%sp}
{%sp}.append {%tn}
group g1 {%forecast1} {%forecast2}
{%sp}.append g1.graph
{%sp}.name untitled01 DMtable
{%sp}.name untitled02 ForecastGraphs
show {%sp}

delete {%forecast1} {%forecast2} {%ld} {%tn} g1 {%fe1} {%fe2}
smpl {%origsmpl}


